require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe Pokan::EventHandler do

  before :each do
    @event_handler = Pokan::EventHandler.new
  end

  it 'should register a new event' do
    register_callback(:dummy)

    @event_handler.events.should include :dummy
  end

  it 'should raise ArgumentError if event name is not a symbol' do
    expect { @event_handler.register 'dummy' }.to raise_exception ArgumentError
  end

  it 'should not allow a call to `register` without a block' do
    expect { @event_handler.register :dummy }.to raise_exception Pokan::NoBlockGivenError
  end

  it 'should list no subscribers for a not registered event' do
    @event_handler.subscribers(:dummy).should be_empty
  end

  it 'should list all subscribers for an event' do
    3.times { register_callback(:dummy) }

    @event_handler.subscribers(:dummy).size.should == 3
  end

  it 'should do nothing when `emit` is called for an unexisting event' do
    @event_handler.emit(:dummy).should == @event_handler
  end

  it 'should call registered callbacks after `emit`' do
    @block_called = false
    register_callback(:dummy)

    @event_handler.emit(:dummy)

    @block_called.should be_true
  end

  it 'should call registered callbacks twice' do
    @block_called = false
    register_callback(:dummy)

    @event_handler.emit(:dummy)
    @block_called.should be_true

    @block_called = false
    @event_handler.emit(:dummy)

    @block_called.should be_true
  end

  it 'should pass correct parameters when `emit` is called using `with` option' do
    @parameter = false

    @event_handler.register :dummy do |message|
      @parameter = message  
    end

    @event_handler.emit :dummy, :with => [true]
    @parameter.should be_true
  end

  it 'should reset registered callbacks for an event' do
    3.times { register_callback(:dummy) }

    @event_handler.reset(:dummy)

    @event_handler.subscribers(:dummy).should be_empty
  end

  def register_callback(event)
    @event_handler.register event do
      @block_called = true
    end
  end

end
