require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe Pokan::Peer do
  context 'when checking the status of the peer' do
    before :all do
      Pokan::Connection.redis.flushall
    end

    it 'should check if a peer is dead' do
      peer = Pokan::Peer.new
      peer.address = peer.udp_port = '3'
      peer.kill
      peer.save
      peer.reload
      peer.dead?.should be_true
    end

    it 'should check if a peer is not dead' do
      peer = Pokan::Peer.new
      peer.address = peer.udp_port = '3'
      peer.save
      peer.reload
      peer.dead?.should be_false
    end

    it 'should check if a peer is not alive' do
      peer = Pokan::Peer.new
      peer.address = peer.udp_port = '3'
      peer.kill
      peer.save
      peer.alive?.should be_false
    end

    it 'should check if peer is alive' do
      peer = Pokan::Peer.new
      peer.address = peer.udp_port = '3'
      peer.save
      peer.alive?.should be_true
    end
  end

  context 'when changing the status of a peer' do
    before :all do
      Pokan::Connection.redis.flushall
    end

    it 'should not set a new status if the timestamp is not greater' do
      peer = Pokan::Peer.new
      peer.address = peer.udp_port = '3'
      peer.store(:status, 'dead', Time.at(0))
      peer.save
      peer.reload
      peer.dead?.should be_false
    end

    it 'should set a new status if the timestamp is greater' do
      peer = Pokan::Peer.new
      peer.address = peer.udp_port = '3'
      peer.store('status', 'dead', Time.now + 1000)
      peer.save
      peer.reload
      peer.dead?.should be_true
    end
  end

  context 'when setting up a peer' do
    before :all do
      Pokan::Connection.redis.flushall
      @peer = Pokan::Peer.new
      @peer.address = @peer.udp_port = '3'
      @peer.save
    end

    it 'should start as a normal peer' do
      @peer.seed?.should be_false
    end

    it 'should start act as a seed' do
      @peer.act_as_seed
      @peer.save
      @peer.seed?.should be_true
    end

    it 'should kill a peer' do
      @peer.kill
      @peer.dead?.should be_true
    end

    it 'should revive a peer' do
      @peer.revive
      @peer.dead?.should be_false
    end
  end
end
