require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe Pokan::Query do
  before :all do
    Pokan::Connection.redis.flushall
    @query = Pokan::Query.new(Pokan::Peer)
    p1 = Pokan::Peer.new
    p1.address = p1.udp_port = '1'
    p1.act_as_seed
    p1.kill
    p1.save
    p2 = Pokan::Peer.new
    p2.address = p2.udp_port = '2'
    p2.act_as_seed
    p2.save
    p3 = Pokan::Peer.new
    p3.address = p3.udp_port = '3'
    p3.save
    p4 = Pokan::Peer.new
    p4.address = p4.udp_port = '4'
    p4.kill
    p4.save
  end

  context 'when getting all peers' do
    it 'should return all alive peers' do
      peers = @query.where(role:'peer', status:'alive').collect { |p| p.id }
      peers.should =~ ['3:3']
    end

    it 'should return all alive seeds' do
      peers = @query.where(role:'seed', status:'alive').collect { |p| p.id }
      peers.should == ['2:2']
    end
  end

  context 'when search a specific peer' do
    it'should return a empty array if it does not exist' do
      peers = @query.where(id:'5:5').collect! { |p| p.id }
      peers.should == []
    end

    it 'should return the peer, if it exists' do
      peers = @query.where(id:'4:4').collect! { |p| p.id }
      peers.should == ['4:4']
    end
  end

  context 'when choosing a peer' do
    it 'should return a random seed' do
      seeds = @query.where(role:'seed').collect! { |p| p.id }
      id = @query.where(random:true, role:'seed')[0].id
      seeds.include?(id).should be_true
    end

    it 'should return a random peer' do
      peers = @query.where(role:'peer').collect! { |p| p.id }
      id = @query.where(random:true, role:'peer')[0].id
      peers.include?(id).should be_true
    end

    it 'should not return the peer with the specific id' do 
      peers = @query.where(:not => ['1:1', '2:2', '3:3', '4:4'])
      peers.include?('1:1').should be_false
    end
  end
end
