require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe Pokan::ServerMessages do
  before :all do
    class Messenger
      include Pokan::ServerMessages
      attr_accessor :address, :port
    end
    @messenger = Messenger.new
    @messenger.port = 'port'
    @messenger.address = 'ip'
    Pokan::RequestHandler.address = 'ip'
    Pokan::RequestHandler.port = 'port'
    @req_handler = Pokan::RequestHandler.new nil
  end

  before :each do
    Pokan::Connection.redis.flushall
    @req_handler.merge({
      p1: {
        status: { value: 'alive', timestamp: 100 },
        role: { value: 'peer', timestamp: 100 },
        answer: { value: 'FALSE', timestamp: 100 },
        foo: { value: 'baz', timestamp: 101 }
      },
      p2: {
        status: { value: 'alive', timestamp: 100 },
        role: { value: 'peer', timestamp: 100 },
        answer: { value: '+/-', timestamp: 102 },
        foo: { value: 'baz', timestamp: 103 }
      }
    })
  end

  context 'when generating a digest message' do
    before :each do
      @digest = JSON.parse(@messenger.digest_message)
    end

    it 'should set action as digest' do
      @digest['action'].should == 'digest'
    end

    it 'should set the timestamps of all keys of all known ids' do
      @digest['data'].should == {
        'p1' => { 'answer' => 100, 'foo' => 101, 'status' => 100, 'role' => 100 },
        'p2' => { 'answer' => 102, 'foo' => 103, 'status' => 100, 'role' => 100 }
      }
    end
  end

  context 'when receiving a hello message' do
    it 'should create the peer and set its role' do
      hello_message = {
        'action' => 'hello',
        'role' => 'peer',
        'origin' => 'ip:port'
      }
      @req_handler.receive_data(hello_message.to_json)
      peer = Pokan::Query.new(Pokan::Peer).where(id:'ip:port')[0]
      peer.value(:role).should == 'peer'
    end
  end

  context 'when receiving a goodbye message' do
    it 'should kill the origin peer' do
      peer = Pokan::Peer.new
      peer.address = 'ip'
      peer.udp_port = 'port'
      peer.save
      goodbye_message = {
        'action' => 'goodbye',
        'timestamp' => 97317585671,
        'origin' => 'ip:port'
      }
      @req_handler.receive_data(goodbye_message.to_json)
      peer = Pokan::Query.new(Pokan::Peer).where(id: 'ip:port')[0]
      peer.value(:status).should == 'dead'
    end
  end
end
