require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe Pokan::Server do

  before :each do
    Pokan::Connection.redis.flushall
  end

  context 'when server is being configured manually' do

    before { @server = Pokan::Server.new('10.10.10.8') }

    it 'should gossip every specified interval' do
      @server.gossip_every(3)
      @server.gossip_interval.should == 3
    end

    it 'should use epoll when it is told to' do
      @server.use_epoll
      @server.using_epoll?.should be_true
    end

  end

  context 'when behavior is added to the server' do

    before { @server = Pokan::Server.new('127.0.0.1') }

    it 'should properly call registered callbacks on events' do
      @block_called = false
      add_behavior_to_event(:new_key)

      @server.store 'k', 'v'

      @block_called.should be_true
    end

    it 'should call registered, periodic behavior' do
      @block_called = false
      @server.every(2) { @block_called = true; @server.stop }

      @server.start
      @block_called.should be_true
    end

    def add_behavior_to_event(event)
      @server.on event do |key, value|
        @block_called = true
      end
    end

  end

end
